<?php 

/**
 * @file
 * Company implementation.
 */

/**
 * The company definition.
 *
 * @return array
 */
function _pagarme_marketplace_company_menu_definition() {
  $items = array();

  $items['admin/commerce/config/marketplace/company'] = array(
    'title' => 'Company',
    'description' => 'Primary recipient settings and information.',
    'page callback' => 'drupal_get_form',
    'page arguments' => array('pagarme_marketplace_company'),
    'access arguments' => array('access company info'),
    'file' => 'includes/pagarme_marketplace.company.inc',
  );

  return $items;
}

function pagarme_marketplace_company($form, $form_state) {
  $pagarme_marketplace = new \Drupal\pagarme_marketplace\PagarmeMarketplace();
  $company_info = $pagarme_marketplace->getCompanyInfo();
  $balance = $pagarme_marketplace->pagarme->balance()->get();

  $form['company'] = array(
    '#type' => 'fieldset', 
    '#title' => t('Balance'),
    '#collapsible' => TRUE, 
    '#collapsed' => FALSE,
  );

  $header = array(
    t('Amount receivable'),
    t('Available value'),
    t('Amount already transferred')
  );

  $rows = array();
  $currency_code = commerce_default_currency();
  $currency = commerce_currency_load($currency_code);

  $waiting_funds = $balance->getWaitingFunds()->amount;
  $rows['data']['waiting_funds'] = pagarme_currency_format($waiting_funds);

  $available = $balance->getAvailable()->amount;
  $rows['data']['available'] =  pagarme_currency_format($available);

  $transferred = $balance->getTransferred()->amount;
  $rows['data']['transferred'] =  pagarme_currency_format($transferred);

  $form['company']['balance'] = array(
    '#theme' => 'table',
    '#header' => $header,
    '#rows' => $rows,
  );

   $form['recipient']['account'] = array(
    '#type' => 'fieldset', 
    '#title' => t('Account information'),
    '#collapsible' => TRUE, 
    '#collapsed' => FALSE,
  );

   // Default Recipient ID Pagarme.me
  $server = variable_get('pagarme_server');
  $default_recipient_id = $company_info->default_recipient_id->{$server};
  $default_recipient = $pagarme_marketplace->pagarme->recipient()->get($default_recipient_id);
  $rows = array();
  $rows[] = array(t('NAME/COMPANY NAME'), $default_recipient->getBankAccount()->getLegalName());
  $rows[] = array(t('BANK'), $default_recipient->getBankAccount()->getBankCode());
  $rows[] = array(t('CPF/CNPJ'), $default_recipient->getBankAccount()->getDocumentNumber());
  $rows[] = array(t('AGENCY'), $default_recipient->getBankAccount()->getAgencia());
  $rows[] = array(t('BANK ACCOUNT'), $default_recipient->getBankAccount()->getConta());

  $form['recipient']['account']['info'] = array(
    '#markup' => theme('table', array('rows' => $rows))
  );
  return $form;
}
