<?php

/**
 * @file
 * Hooks provided by the pagarme_marketplace module.
 */

/**
 * Perform alterations to a structured query that gets the split of a product.
 * By registering multiple division rules for the same product, the most recent rule will be used by default. To use a specific one in a given context use that hook. When you get the split of a product, additional metadata is added to the query that can then be used to decide if and how to take action.
 * 
 * @param $query
 *   An Query object describing the composite parts of a SQL query.
 */
function hook_query_load_split_product_alter(QueryAlterableInterface $query) {
  $metadata = $query->getMetaData('metadata');
  if (!empty($metadata['commerce_line_item'])) {
    $commerce_line_item = $metadata['commerce_line_item'];

    $entity = $commerce_line_item->data['context']['entity'];
    $node = node_load($entity['entity_id']);
    $node_wrapper = entity_metadata_wrapper('node', $node);
    $recipient_id = $node_wrapper->field_recipient_id->value();

    if ($recipient_id) {
      $query->leftJoin('pagarme_split_rules', 'rules', 'rules.split_id = splits.split_id');
      $query->condition('rules.recipient_id', $recipient_id);
    }
  }
}
