<?php

namespace Drupal\pagarme\Controllers\FormControllers;

class PagarmeConfig extends \Drupal\cool\BaseSettingsForm {

  static public function getId() {
    return 'pagarme_settings_form';
  }

  static public function build() {
    $form = parent::build();
    $config = \Drupal\pagarme\Helpers\PagarmeUtility::getPagarmeConfig();
    $default_currency = commerce_default_currency();

    $form['basic_settings'] = array(
      '#type' => 'fieldset',
      '#title' => t('Basic configurations')
    );
    $form['basic_settings']['pagarme_server'] = array(
      '#type' => 'select',
      '#title' => t('Pagar.me server'),
      '#options' => array(
        'test' => t('TEST'),
        'live' => t('LIVE'),
      ),
      '#description' => t('TEST - Use it to test, LIVE - Use it to process real transactions'),
      '#default_value' => $config['pagarme_server'],
      '#required' => TRUE,
    );
    $form['basic_settings']['pagarme_api_key'] = array(
      '#type' => 'textfield',
      '#title' => t('API Key'),
      '#description' => t('API Key (available on your dashboard)'),
      '#default_value' => $config['pagarme_api_key'],
      '#required' => TRUE,
    );
    $form['basic_settings']['pagarme_encryption_key'] = array(
      '#type' => 'textfield',
      '#title' => t('Crypt Key'),
      '#description' => t('Crypt key (available on your dashboard)'),
      '#default_value' => $config['pagarme_encryption_key'],
      '#required' => TRUE,
    );
    $form['basic_settings']['pagarme_display_title'] = array(
      '#type' => 'textfield',
      '#title' => t('Title displayed on payment method'),
      '#description' => t('Text used on payment page as description for payment method.'),
      '#default_value' => $config['pagarme_display_title'],
    );
    $form['basic_settings']['pagarme_display_title_pay_button'] = array(
      '#type' => 'textfield',
      '#title' => t('Title displayed on payment button'),
      '#description' => t('Text used on payment button (Payment page).'),
      '#default_value' => $config['pagarme_display_title_pay_button'],
    );
    $form['basic_settings']['pagarme_ui_color'] = array(
      '#type' => 'textfield',
      '#title' => t('Primary color of Checkout user interface'),
      '#default_value' => $config['pagarme_ui_color'],
      '#description' => t('Primary color of Checkout user interface'),
      '#required' => TRUE,
    );
    $form['basic_settings']['pagarme_async_transaction'] = array(
      '#type' => 'checkbox',
      '#title' => t('Asynchronous transaction processing'),
      '#description' => t('Leave this option unchecked if you want to use POSTbacks and keep synchronous processing of a transaction.'),
      '#default_value' => $config['pagarme_async_transaction'],
    );
    $form['basic_settings']['pagarme_debug'] = array(
      '#type' => 'checkbox',
      '#title' => t('Enable debug for each action of this module'),
      '#default_value' => $config['pagarme_debug'],
    );
    $form['advanced_settings'] = array(
      '#type' => 'fieldset',
      '#title' => t('Advanced options')
    );
    $form['advanced_settings']['general'] = array(
      '#type' => 'fieldset',
      '#title' => t('General options'),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE
    );
    $general_settings = &$form['advanced_settings']['general'];
    $general_settings['pagarme_checkout_complete_url'] = array(
      '#type' => 'textfield',
      '#title' => t('Redirect URL after complete an transaction'),
      '#description' => t('Redirect URL after complete an transaction (complete checkout).'),
      '#default_value' => $config['pagarme_checkout_complete_url'],
      '#required' => TRUE,
    );
    $general_settings['pagarme_checkout_failure_url'] = array(
      '#type' => 'textfield',
      '#title' => t('Redirect URL of an transaction that can not be processed by Pagar.me'),
      '#description' => t('Redirect URL of an transaction that can not be processed by Pagar.me'),
      '#default_value' => $config['pagarme_checkout_failure_url'],
      '#required' => TRUE,
    );
    if (module_exists('token')) {
      $general_settings['tokens'] = array(
        '#theme' => 'token_tree',
        '#token_types' => array('commerce-order'),
      );
    }
    $form['advanced_settings']['order_display'] = array(
      '#type' => 'fieldset',
      '#title' => t('Order settings'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE
    );
    if (module_exists('token')) {
    $order_display_settings = &$form['advanced_settings']['order_display'];
      // $order_display_settings['pagarme_order_reason_token'] = array(
      //   '#type' => 'textfield',
      //   '#title' => t('Token de pedido'),
      //   '#description' => t('Qual o título que você deseja usar para os pedidos enviados para a Pagar.me. O padrão é "Pedido @order_number em @store". <strong>Você pode usar tokens</strong> aqui(veja o navegador de tokens abaixo).'),
      //   '#default_value' => $config['pagarme_order_reason_token'],
      // );

      $order_display_settings['pagarme_street_number_token'] = array(
        '#type' => 'textfield',
        '#title' => t('Residence address token'),
        '#description' => t('From where you want to get the "residence number" field to sent on Pagar.me. <strong>You can use tokens</strong> here(see token browser below).'),
        '#default_value' => $config['pagarme_street_number_token'],
      );
      $order_display_settings['pagarme_phone_ddd_token'] = array(
        '#type' => 'textfield',
        '#title' => t('Area code of the phone'),
        '#description' => t('From where you want to get "Area code of the phone" field to sent on Pagar.me. <strong>You can use tokens</strong> here(see token browser below).'),
        '#default_value' => $config['pagarme_phone_ddd_token'],
      );
      $order_display_settings['pagarme_phone_number_token'] = array(
        '#type' => 'textfield',
        '#title' => t('Phone number token'),
        '#description' => t('From where you want to get "Phone" field to sent on Pagar.me. <strong>You can use tokens</strong> here(see token browser below).'),
        '#default_value' => $config['pagarme_phone_number_token'],
      );
      // $order_display_settings['pagarme_name_token'] = array(
      //   '#type' => 'textfield',
      //   '#title' => t('Token de nome'),
      //   '#description' => t('De onde você deseja obter os dados de campo "Nome" para enviar para Pagar.me. <strong>Você pode usar tokens</strong> aqui(veja o navegador de tokens abaixo).'),
      //   '#default_value' => $config['pagarme_name_token'],
      // );
      $order_display_settings['pagarme_cpf_token'] = array(
        '#type' => 'textfield',
        '#title' => t('CPF Token'),
        '#description' => t('From where you want to get "CPF" field to sent on Pagar.me. <strong>You can use tokens</strong> here(see token browser below).'),
        '#default_value' => $config['pagarme_cpf_token'],
      );
      $order_display_settings['pagarme_cnpj_token'] = array(
        '#type' => 'textfield',
        '#title' => t('CNPJ Token'),
        '#description' => t('From where you want to get "CNPJ" field to sent on Pagar.me. <strong>You can use tokens</strong> here(see token browser below).'),
        '#default_value' => $config['pagarme_cnpj_token'],
      );
      $order_display_settings['pagarme_birthday_token'] = array(
        '#type' => 'textfield',
        '#title' => t('Birthday token'),
        '#description' => t('From where you want to get "Birthday" field to sent on Pagar.me. <strong>You can use tokens</strong> here(see token browser below).'),
        '#default_value' => $config['pagarme_birthday_token'],
      );
      $order_display_settings['pagarme_gender_token'] = array(
        '#type' => 'textfield',
        '#title' => t('Gender Token'),
        '#description' => t('From where you want to get "Gender" field to sent on Pagar.me. <strong>You can use tokens</strong> here(see token browser below).'),
        '#default_value' => $config['pagarme_gender_token'],
      );
      $order_display_settings['tokens'] = array(
        '#theme' => 'token_tree',
        '#token_types' => array('commerce-order'),
      );
    }
    $form['advanced_settings']['payment_settings'] = array(
      '#type' => 'fieldset',
      '#title' => t('Payment configurations'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE
    );
    $payment_settings = &$form['advanced_settings']['payment_settings'];
    $payment_settings['pagarme_customer_data'] = array(
      '#type' => 'checkbox',
      '#title' => t('Catch customer data on the Checkout?'),
      '#description' => t('In case you do not want to catch customer data on the Checkout, disable this option'),
      '#default_value' => $config['pagarme_customer_data'],
    );
    $payment_settings['pagarme_disable_zero_document_number'] = array(
      '#type' => 'checkbox',
      '#title' => t('Dot not accept if either CPF or CNPJ are zeros?'),
      '#default_value' => $config['pagarme_disable_zero_document_number'],
      '#states' => array(
        'visible' => array('input[name="pagarme_customer_data"]' => array('checked' => TRUE)),
      ),
    );

    $config['pagarme_payment_methods']['credit_card'] = 'credit_card';
    $payment_settings['pagarme_payment_methods'] = array(
      '#type' => 'checkboxes',
      '#title' => t('Payment methods available on the Checkout'),
      '#description' => t('Payment methods available on the Checkout.'),
      '#options' => array('credit_card' => t('credit card'), 'boleto' => t('billet')),
      '#default_value' => $config['pagarme_payment_methods'],
    );
    $payment_settings['pagarme_payment_methods']['credit_card']['#disabled'] = TRUE;

    $payment_settings['pagarme_max_installments'] = array(
      '#type' => 'select',
      '#title' => t('Maximum number of plots'),
      '#description' => t('Maximum accepted number of plots.'),
      '#default_value' => $config['pagarme_max_installments'],
      '#options' => \Drupal\pagarme\Helpers\PagarmeUtility::installmentsNumber(),
      '#states' => array(
        'visible' => array('input[name="pagarme_payment_methods[credit_card]"]' => array('checked' => TRUE)),
      ),
    );
    $payment_settings['pagarme_default_installment'] = array(
      '#type' => 'select',
      '#title' => t('Default plot on cart open'),
      '#description' => t('Default plot on cart open.'),
      '#default_value' => $config['pagarme_default_installment'],
      '#options' => \Drupal\pagarme\Helpers\PagarmeUtility::installmentsNumber(),
      '#states' => array(
        'visible' => array('input[name="pagarme_payment_methods[credit_card]"]' => array('checked' => TRUE)),
      ),
    );
    $payment_settings['pagarme_interest_rate'] = array(
      '#type' => 'textfield',
      '#title' => t('Interest rate to be charged to the transaction'),
      '#description' => t('Interest rate to be charged to the transaction.'),
      '#default_value' => $config['pagarme_interest_rate'],
      '#element_validate' => array('element_validate_number'),
      '#size' => 15,
      '#states' => array(
        'visible' => array('input[name="pagarme_payment_methods[credit_card]"]' => array('checked' => TRUE)),
      ),
    );

    $pagarme_installment_start_value = commerce_currency_amount_to_decimal($config['pagarme_installment_start_value'], $default_currency);
    $payment_settings['pagarme_installment_start_value'] = array(
      '#type' => 'textfield',
      '#title' => t('Enable installment above'),
      '#description' => t('Enable the installment for orders with value higher the from informed.'),
      '#default_value' => $pagarme_installment_start_value,
      '#element_validate' => array('element_validate_number'),
      '#size' => 15,
      '#states' => array(
        'visible' => array('input[name="pagarme_payment_methods[credit_card]"]' => array('checked' => TRUE)),
      ),
    );
    $payment_settings['pagarme_free_installments'] = array(
      '#type' => 'select',
      '#title' => t('Number of plots that do not have interest rate charged'),
      '#description' => t('Number of plots that do not have interest rate charged.'),
      '#default_value' => $config['pagarme_free_installments'],
      '#options' => \Drupal\pagarme\Helpers\PagarmeUtility::installmentsNumber(),
      '#states' => array(
        'visible' => array('input[name="pagarme_payment_methods[credit_card]"]' => array('checked' => TRUE)),
      ),
    );
    $payment_settings['pagarme_card_brands'] = array(
      '#type' => 'checkboxes',
      '#title' => t('Flags accepted by Checkout'),
      '#description' => t('Flags accepted by Checkout'),
      '#options' => \Drupal\pagarme\Helpers\PagarmeUtility::cardBrands(),
      '#default_value' => $config['pagarme_card_brands'],
      '#states' => array(
        'visible' => array('input[name="pagarme_payment_methods[credit_card]"]' => array('checked' => TRUE)),
      ),
    );

    $payment_settings['pagarme_boleto_discount'] = array(
      '#type' => 'select',
      '#title' => t('Discount on billet'),
      '#description' => t('Select the type of discount for the billet'),
      '#options' => array('_none' => t('None'), 'amount' => t('In cents'), 'percentage' => t('In percent')),
      '#default_value' => $config['pagarme_boleto_discount'],
      // '#ajax' => array(
      //   'callback' => array(new \Drupal\pagarme\Controllers\FormControllers\PagarmeConfig(), '_pagarme_boleto_discount_ajax_callback'),
      //   'method' => 'replace',
      //   'effect' => 'fade',
      // ),
      '#states' => array(
        'visible' => array('input[name="pagarme_payment_methods[boleto]"]' => array('checked' => TRUE)),
      ),
    );

    $pagarme_boleto_discount_amount = commerce_currency_amount_to_decimal($config['pagarme_boleto_discount_amount'], $default_currency);
    $payment_settings['pagarme_boleto_discount_amount'] = array(
      '#type' => 'textfield',
      '#title' => t('Discount amount'),
      '#description' => t('Value of the discount if the means of payment is a ticket. Ex: discount of R$ 10.25 = 10.25.'),
      '#default_value' => $pagarme_boleto_discount_amount,
      '#element_validate' => array('element_validate_number'),
      '#prefix' => '<div id="boleto-discount-amount-options-replace">',
      '#suffix' => '</div>',
      '#states' => array(
        'visible' => array(
          'input[name="pagarme_payment_methods[boleto]"]' => array('checked' => TRUE),
          'select[name="pagarme_boleto_discount"]' => array('value' => 'amount'),
        ),
      ),
    );
    $payment_settings['pagarme_boleto_discount_percentage'] = array(
      '#type' => 'textfield',
      '#title' => t('Discount amount'),
      '#description' => t('Discount percent if the payment method be billet. Example: discount of 25% = 25.'),
      '#default_value' => $config['pagarme_boleto_discount_percentage'],
      '#element_validate' => array('element_validate_integer_positive'),
      '#prefix' => '<div id="boleto-discount-percentage-options-replace">',
      '#suffix' => '</div>',
      '#maxlength' => 2,
      '#size' => 4,
      '#states' => array(
        'visible' => array(
          'input[name="pagarme_payment_methods[boleto]"]' => array('checked' => TRUE),
          'select[name="pagarme_boleto_discount"]' => array('value' => 'percentage'),
        ),
      ),
    );

    $pagarme_boleto_discount_start = commerce_currency_amount_to_decimal($config['pagarme_boleto_discount_start'], $default_currency);
    $payment_settings['pagarme_boleto_discount_start'] = array(
      '#type' => 'textfield',
      '#title' => t('Apply discount from'),
      '#description' => t('Discount will be applied only in orders that the amount is higher than the informed amount. Example: Apply discount from R$999,25  = 999.25.'),
      '#default_value' => $pagarme_boleto_discount_start,
      '#element_validate' => array('element_validate_number'),
      '#prefix' => '<div id="boleto-discount-amount-options-replace">',
      '#suffix' => '</div>',
      '#states' => array(
        'visible' => array(
          array(
            'input[name="pagarme_payment_methods[boleto]"]' => array('checked' => TRUE),
            'select[name="pagarme_boleto_discount"]' => array('value' => 'amount'),
          ),
          'xor',
          array(
            'input[name="pagarme_payment_methods[boleto]"]' => array('checked' => TRUE),
            'select[name="pagarme_boleto_discount"]' => array('value' => 'percentage'),
          ),
        ),
      ),
    );
    $payment_settings['pagarme_boleto_helper_text'] = array(
      '#type' => 'textarea',
      '#title' => t('Optional message for billet'),
      '#description' => t('Optional message that will appear below the payment button of billet method.'),
      '#default_value' => $config['pagarme_boleto_helper_text'],
      '#states' => array(
        'visible' => array('input[name="pagarme_payment_methods[boleto]"]' => array('checked' => TRUE)),
      ),
    );
    $payment_settings['pagarme_credit_card_helper_text'] = array(
      '#type' => 'textarea',
      '#title' => t('Credit card optional message'),
      '#description' => t('Optional message that will appear below the payment button of credit card payment method.'),
      '#default_value' => $config['pagarme_credit_card_helper_text'],
      '#states' => array(
        'visible' => array('input[name="pagarme_payment_methods[credit_card]"]' => array('checked' => TRUE)),
      ),
    );
    return $form;
  }

  public function _pagarme_boleto_discount_ajax_callback($form, &$form_state) {
    $commands = array();
    $commands[] = ajax_command_replace(
        '#boleto-discount-amount-options-replace',
        render($form['advanced_settings']['payment_settings']['pagarme_boleto_discount_amount'])
    );
    $commands[] = ajax_command_replace(
        '#boleto-discount-percentage-options-replace',
        render($form['advanced_settings']['payment_settings']['pagarme_boleto_discount_percentage'])
    );
    return array('#type' => 'ajax','#commands' => $commands);
  }

  static public function validate($form, &$form_state) {
    $form_state['values']['pagarme_payment_methods']['credit_card'] = 'credit_card';
    $values =  &$form_state['values'];
    $values['pagarme_installment_start_value'] = commerce_currency_decimal_to_amount($values['pagarme_installment_start_value'], commerce_default_currency());
    if (!count(array_filter($values['pagarme_payment_methods']))) {
      form_set_error('pagarme_payment_methods', t('Available payment methods on the checkout. Is required to enable a minimum of one payment method.'));
    }

    if ($values['pagarme_payment_methods']['credit_card']) {
      if (!count(array_filter($values['pagarme_card_brands']))) {
        form_set_error('pagarme_card_brands', t('Credit card company flags accepted on the Checkout. Is required to enable a minimum of one company flag.'));
      }
    }
    if ($values['pagarme_payment_methods']['boleto']) {
      switch ($values['pagarme_boleto_discount']) {
        case 'amount':
          $discount_amount = &$values['pagarme_boleto_discount_amount'];
          if (empty($discount_amount)) {
            form_set_error('pagarme_boleto_discount_amount', t('Field discount amount is required.'));
          } else {
            $discount_start = &$values['pagarme_boleto_discount_start'];
            if (empty($discount_start)) {
              form_set_error('pagarme_boleto_discount_start', t('Field "Apply Discount From" is required.'));
            } else {

              $discount_amount = commerce_currency_decimal_to_amount($discount_amount, commerce_default_currency());
              $discount_start = commerce_currency_decimal_to_amount($discount_start, commerce_default_currency());

              if ($discount_amount >= $discount_start) {
                form_set_error('pagarme_boleto_discount_amount', t('The discount amount are either higher than or equal the minimum discount value.'));
              }
            }
          }
          break;
        case 'percentage':
          if (empty($values['pagarme_boleto_discount_percentage'])) {
            form_set_error('pagarme_boleto_discount_percentage', t('Field "Discount Percent" is required.'));
          }
          break;
      }
    }
  }
}
